import json

from odoo import http


class MrpBomDump(http.Controller):
    @http.route('/mrp/bom/dump', type='http', auth='user')
    def index(self, **kw):
        bom = http.request.env['mrp.bom'].search([('id', '=', kw.get('id'))])
        if bom.status == 'external':
            return self.bom2(**kw)
        elif bom.status == 'internal':
            return self.bom1(**kw)
        return ''

    def bom1(self, **kw):
        res = {}
        bom = http.request.env['mrp.bom'].search([('id', '=', kw.get('id'))])
        res["bom_status"] = bom.status
        res["project"] = bom.project_id.name
        res["inquiry_code"] = bom.inquiry_id.code
        res["date"]  = str(bom.date)
        res["engineer"] = bom.user_id.name
        res["bom_lines"] = []
        for line in bom.bom_line_ids:
            res["bom_lines"].append({
                "product": line.product_id.name,
                "qty": line.product_qty,
                "price": line.price_unit,
            })
        return json.dumps(res)

    def bom2(self, **kw):
        res = {}
        bom = http.request.env['mrp.bom'].search([('id', '=', kw.get('id'))])
        inquiry = bom.inquiry_id
        res['bom_status'] = bom.status
        res['order_code'] = inquiry.code
        res['name'] = bom.product_tmpl_id.name
        res['customer'] = bom.customer_id.name
        res['date'] = str(bom.date)
        res['engineer'] = bom.user_id.name
        res['bom_lines'] = []
        for line in bom.bom_line_ids:
            suppliers = []
            purchase = (line.material_source == 'stock' and line.product_id.qty_available < line.product_qty) or line.material_source == 'purchase'
            for supplier in sorted(line.product_id.seller_ids, key=lambda s:s['price'])[:3]:
                suppliers.append({
                    'name': supplier.name.name,
                    'price': supplier.price,
                    #'currency': supplier.currency_id.name,
                    'moq': supplier.min_qty,
                    'lead_time': supplier.delay,
                })
            res['bom_lines'].append({
                'purchase': purchase,
                'product': line.product_id.name,
                'qty': line.product_qty,
                'suppliers': suppliers,
                'best_supplier': min(suppliers, key=lambda s: s['price']) if len(suppliers) else ''
            })
        return json.dumps(res)
