#  -*- coding: utf-8 -*-

from odoo import api, fields, models


class Bom(models.Model):
    _inherit = "mrp.bom"

    status = fields.Selection([
        ('internal', 'bom1'),
        ('quotation', 'bom1-quotation'),
        ('external', 'bom2')
    ], string='bom 類型', default='external', tracking=True)

    amount_total = fields.Float(string='總價', compute='_compute_total', store=True)
    quotation_total = fields.Float(string='總報價', compute='_compute_quotation', store=True)

    project_id = fields.Many2one('project.project', string='專案', tracking=True, default=False)
    inquiry_id = fields.Many2one('sales.inquiry', string='廠勘表', tracking=True, default=False)
    purchase_id = fields.Many2one('purchase.internal.order', string='採購')

    date = fields.Date(string='日期', default=fields.Datetime.now, tracking=True)
    user_id = fields.Many2one('res.users', string='工程師', tracking=True, default=lambda self: self.env.user)
    customer_id = fields.Many2one('res.partner', string='客戶')

    @api.onchange('user_id')
    def _assign_engineer(self):
        # notification_ids = [((0, 0, {
        #     'res_partner_id': self.env.user.partner_id.id,
        #     'notification_type': 'inbox'}))]
        url = f'/web#id={self.id}&view_type=form&model=mrp.bom'
        bom_name = self.product_tmpl_id.name
        # user_id = self.env.user.id
        message = f"已被指派為 BOM <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{bom_name}</a> 工程師"
        # self.user_id.partner_id.message_post(author_id=user_id,
        #                          body=message + "請查收",
        #                          message_type='notification',
        #                          subtype_xmlid="mail.mt_comment",
        #                          notification_ids=notification_ids,
        #                          partner_ids=[self.env.user.id],
        #                          notify_by_email=False,
        #                          )
        email_values = {
            'email_to': self.user_id.partner_id.email,
            'email_cc': self.env.user.partner_id.email,
            'email_from': "erp_system@kci-global.com.tw",
            'subject': '[KCI ERP System] BOM 通知',
            'body_html': message,
        }
        print('----------------------')
        print(self.user_id.partner_id.email)
        mail_template = self.env.ref('kci-mrp.bom_email_template')
        mail_template.send_mail(self.ids[0], force_send=True, email_values=email_values)

    def write(self, vals):
        if self.status == 'internal':
            self.action_send_bom_notification(bom_name=self.product_tmpl_id.name,
                                              is_create=False,
                                              url=f'/web#id={self.id}&view_type=form&model=mrp.bom')
        return super(Bom, self).write(vals)

    def action_bom1_to_quotation(self):
        new_bom = self.copy(default={'status': 'quotation'})
        self.action_send_bom_notification(bom_name=new_bom.product_tmpl_id.name, 
                                          is_create=True,
                                          url=f'/web#id={new_bom.id}&view_type=form&model=mrp.bom')

    def action_create_quotation(self):
        quotation = self.env['sale.order'].create({
            'name': self.product_tmpl_id.name,
            'inquiry_id': self.inquiry_id.id,
            'order_code': self.inquiry_id.code,
            'partner_id': self.customer_id.id,
        })
        for line in self.bom_line_ids:
            self.env['sale.order.line'].create({
                'order_id': quotation.id,
                'product_id': line.product_id.id,
                'product_uom_qty': line.product_qty,
                'price_unit': line.quotation_price,
            })
        self.action_send_quotation_notification(quotation.name, self.inquiry_id.sales_person.user_id.partner_id.id, f'/web#id={quotation.id}&view_type=form&model=sale.order')

    def send_notification_to_self(self, message):
        notification_ids = [((0, 0, {
            'res_partner_id': self.env.user.partner_id.id,
            'notification_type': 'inbox'}))]
        user_id = self.env.user.id
        self.env['res.partner'].browse(self.env.user.partner_id.id).message_post(author_id=user_id,
                          body=message + "已寄送",
                          message_type='notification',
                          subtype_xmlid="mail.mt_comment",
                          notification_ids=notification_ids,
                          partner_ids=[self.env.user.id],
                          notify_by_email=False,
                          )

    def action_send_quotation_notification(self, quotation_name, target_partner_id, url):
        # notification_ids = [((0, 0, {
        #     'res_partner_id': self.env.user.partner_id.id,
        #     'notification_type': 'inbox'}))]
        # user_id = self.env.user.id
        message = f"已建立報價單 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{quotation_name}</a> "
        target_user = self.env['res.partner'].browse(target_partner_id)
        # target_user.message_post(author_id=user_id,
        #                   body=message + "請查收",
        #                   message_type='notification',
        #                   subtype_xmlid="mail.mt_comment",
        #                   notification_ids=notification_ids,
        #                   partner_ids=[self.env.user.id],
        #                   notify_by_email=False,
        #                   )
        # self.send_notification_to_self(message)
        email_values = {
            'email_to': target_user.email,
            'email_cc': self.env.user.partner_id.email,
            'email_from': "erp_system@kci-global.com.tw",
            'subject': '[KCI ERP System] 報價單通知',
            'body_html': message,
        }
        mail_template = self.env.ref('kci-mrp.bom_email_template')
        mail_template.send_mail(self.id, force_send=True, email_values=email_values)

    def action_send_bom_notification(self, bom_name, is_create, url):
        # notification_ids = [((0, 0, {
        #     'res_partner_id': self.env.user.partner_id.id,
        #     'notification_type': 'inbox'}))]
        # user_id = self.env.user.id
        message = f"已{'建立' if is_create else '更新'} BOM <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{bom_name}</a> "
        group = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('sales.kci_sales_manager_group' if is_create else 'kci-mrp.kci_mrp_manager_group').id)])
        # group.message_post(author_id=user_id,
        #                    body=message + "請查收",
        #                    message_type='notification',
        #                    subtype_xmlid="mail.mt_comment",
        #                    notification_ids=notification_ids,
        #                    partner_ids=[self.env.user.id],
        #                    notify_by_email=False,
        #                    )
        # self.send_notification_to_self(message)
        email_to = ''
        for partner in group.channel_partner_ids:
            email_to += f'{partner.email}, '
        email_values = {
            'email_to': email_to,
            'email_cc': self.env.user.partner_id.email,
            'email_from': "erp_system@kci-global.com.tw",
            'body_html': message,
        }
        mail_template = self.env.ref('kci-mrp.bom_email_template')
        mail_template.send_mail(self.id, force_send=True, email_values=email_values)

    @api.depends('bom_line_ids.price_subtotal')
    def _compute_total(self):
        for bom in self:
            amount = 0.0
            for line in bom.bom_line_ids:
                amount += line.price_subtotal
            bom.update({
               'amount_total': amount,
            })

    @api.depends('bom_line_ids.quotation_price', 'bom_line_ids.product_qty')
    def _compute_quotation(self):
        for bom in self:
            bom.update({
                'quotation_total': sum(line.quotation_price * line.product_qty for line in bom.bom_line_ids),
            })


class BomLine(models.Model):
    _inherit = 'mrp.bom.line'

    bom_id = fields.Many2one(
        'mrp.bom', 'Parent BoM',
        index=True, ondelete='cascade', required=True)

    price_unit = fields.Float(string='成本', store=True)
    price_subtotal = fields.Float(string='成本小計', store=True, compute='_compute_price_subtotal')
    quotation_price = fields.Float(string='報價', store=True)
    quotation_name = fields.Char(string='報價名稱')
    quotation_subtotal = fields.Float(string='報價小計', store=True, compute='_compute_quotation_subtotal')
    qty_available = fields.Float(string='庫存數量', related='product_id.qty_available', store=True)
    product_type = fields.Selection(related='product_tmpl_id.type', string='產品類型', store=True)
    material_source = fields.Selection([
        ('purchase', '採購'),
        ('stock', '庫存')
    ], string='來源', default='stock')

    @api.depends('quotation_price', 'product_qty')
    def _compute_quotation_subtotal(self):
        for line in self:
            line.quotation_subtotal = line.quotation_price * line.product_qty
 
    @api.depends('product_qty', 'price_unit')
    def _compute_price_subtotal(self):
        for line in self:
            line.price_subtotal = line.product_qty * line.price_unit


# class InternalBom(models.Model):
#     _name = "mrp.internal.bom"
#     _description = "Bom for internal"
#     _inherit = "mrp.bom"

    # bom_line_ids = fields.One2many('mrp.internal.bom.line', 'bom_id', 'BoM Lines', copy=True)
    # byproduct_ids = fields.One2many('mrp.internal.bom.byproduct', 'bom_id', 'By-products', copy=True)


# class InternalBomLine(models.Model):
#     _name = "mrp.internal.bom.line"
#     _order = "sequence, id"
#     _rec_name = "product_id"
#     _description = 'Bill of Material Line'
#     _check_company_auto = True
#     _inherit = "mrp.bom.line"

    # bom_id = fields.Many2one(
    #     'mrp.internal.bom', 'Parent BoM',
    #     index=True, ondelete='cascade', required=True)
    #
    # child_bom_id = fields.Many2one(
    #     'mrp.internal.bom', 'Sub BoM', compute='_compute_child_bom_id')
    # child_line_ids = fields.One2many(
    #     'mrp.internal.bom.line', string="BOM lines of the referred bom",
    #     compute='_compute_child_line_ids')


# class InternalBomByProduct(models.Model):
#     _name = "mrp.internal.bom.byproduct"
#     _description = 'Byproduct'
#     _rec_name = "product_id"
#     _check_company_auto = True
#     _order = 'sequence, id'
#     _inherit = "mrp.bom.byproduct"

    # bom_id = fields.Many2one('mrp.internal.bom', 'BoM', ondelete='cascade', index=True)
