# -*- coding: utf-8 -*-

from datetime import datetime

from odoo import api, fields, models, _


class MrpDrawing(models.Model):
    _name = 'mrp.drawing'
    _description = 'Product Drawing'
    _inherit = ['mail.thread', 'mail.activity.mixin']
    _rec_name = 'product_tmpl_id'
    # _order = "sequence, id"
    # _check_company_auto = True

    code = fields.Char(string='編號', tracking=True)
    name = fields.Char(string='名稱', size=100, tracking=True)
    version = fields.Char(string='版本', size=100, tracking=True)
    description = fields.Char(string='說明', size=1000, tracking=True)
    material_id = fields.Many2one('mrp.drawing.material', string='材質')
    product_id = fields.Many2one(
        'product.product', string='料件',
        check_company=False, index=True, required=True)
    product_tmpl_id = fields.Many2one(
        'product.template', 'Product Template', related='product_id.product_tmpl_id',
        check_company=False, index=True)
    company_id = fields.Many2one(
        'res.company', 'Company', index=True,
        default=lambda self: self.env.company)

    drawing_line_ids = fields.One2many('mrp.drawing.line', 'drawing_id', 'Drawing Lines', copy=True, tracking=True)

    upload_date = fields.Datetime(string='上傳時間', default=fields.Datetime.now, readonly=True)
    last_update_date = fields.Datetime(string='最後更新時間', default=fields.Datetime.now, readonly=False)

    @api.onchange('code', 'name', 'version', 'description', 'product_id', 'product_tmpl_id', 'company_id', 'drawing_line_ids')
    def _refresh_update_date(self):
        self.last_update_date = datetime.now()


class MrpDrawingLine(models.Model):
    _name = 'mrp.drawing.line'
    _description = 'Mrp Drawing Line'
    _inherit = ['mail.thread', 'mail.activity.mixin']
    # _order = "sequence, id"
    # _rec_name = "product_id"
    # _check_company_auto = True

    company_id = fields.Many2one(
        'res.company', 'Company', index=True,
        default=lambda self: self.env.company)

    name = fields.Char(size=100, string='名稱')
    description = fields.Char(size=1000, string='說明')
    drawing_id = fields.Many2one('mrp.drawing', 'Parent Drawing', index=True, ondelete='cascade', required=True, default=False)
    drawing_file = fields.Binary(string='檔案', tracking=True)
    drawing_file_name = fields.Char('File Name')

    upload_date = fields.Datetime(string='上傳時間', default=fields.Datetime.now, readonly=True)

    # @api.model
    # def create(self, vals):
    #     res = super(MrpDrawingLine, self).create(vals)
    #     if vals:
    #         message = "Change info"
    #         res.drawing_id.message_post(message)
    #     return res


class MrpDrawingMaterial(models.Model):
    _name = 'mrp.drawing.material'
    _description = 'Material for Drawing'

    name = fields.Char('材質')
