from datetime import datetime

from odoo import api, fields, models


class PurchaseOrder(models.Model):
    _inherit = ['purchase.order', ]

    contract_department = fields.Char(string='聯絡人部門', size=100)
    contract_name = fields.Char(string='聯絡人姓名', size=100)
    contract_email = fields.Char(string='聯絡人電子信箱', size=100)
    contract_phone = fields.Char(string='聯絡人電話', size=100)

    @api.onchange('partner_id')
    def _refresh_supply_data(self):
        self.update({
            'contract_name': self.partner_id.name,
            'contract_department': self.partner_id.department,
            'contract_email': self.partner_id.email,
            'contract_phone': self.partner_id.phone,
        })

    @api.model
    def create(self, vals):
        company_id = vals.get('company_id', self.default_get(['company_id'])['company_id'])
        self_comp = self.with_company(company_id)
        if vals.get('name', 'New') == 'New':
            seq_date = None
            if 'date_order' in vals:
                seq_date = fields.Datetime.context_timestamp(self, fields.Datetime.to_datetime(vals['date_order']))
            date = datetime.now()
            employee_rec = self.env['hr.employee'].search([('user_id', '=', self.env.uid)], limit=1)
            vals['name'] = f"PO{str(date.year)[2:]}{date.month if date.month > 9 else '0' + str(date.month)}-{str(self_comp.env['ir.sequence'].next_by_code('purchase.order', sequence_date=seq_date))[3:] or '/'}-{employee_rec.sales_id if employee_rec.sales_id else 'S00'}"
        vals, partner_vals = self._write_partner_values(vals)
        res = super(PurchaseOrder, self_comp).create(vals)
        if partner_vals:
            res.sudo().write(partner_vals)  # Because the purchase user doesn't have write on `res.partner`
        res.button_confirm()
        return res
