/** @odoo-module **/

import { browser } from "@web/core/browser/browser";

$(document).on("click", "#report_xlsx", reportXlsx);

const modelMapping = {
    "purchase.order": {
        fetchData: purchaseFetch,
        fillBook: purchase,
        path: "purchase",
    },
    "mrp.bom": {
        fetchData: bomFetch,
        fillBook: bom,
        path: "",
    },
    "sale.order": {
        fetchData: quotationFetch,
        fillBook: quotation,
        path: "quotation",
    },
};

async function quotationFetch() {
    const url = new URL(browser.location.href.replace("#", "?"));
    const id = url.searchParams.get("id");
    const res = await fetch(`/sale/quotation/dump?id=${id}`);
    const data = await res.json();
    return data;
}

async function quotation(workbook, data) {
    const sheet = workbook.sheet(0);
    const date = new Date();
    sheet
        .cell("H4")
        .value(
            `${date.getFullYear()}年${date.getMonth() + 1}月${date.getDate()}日`
        );
    sheet.cell("H5").value(data.inquiry_code);
    sheet.cell("C8").value(data.company);
    sheet.cell("C9").value(data.contract_name);
    sheet.cell("C10").value(data.contract_mobile);
    sheet.cell("C11").value(data.contract_phone);
    sheet.cell("C12").value(data.contract_email);
    sheet.cell("C13").value(data.company_vat);
    sheet
        .range("C14:C15")
        .value(data.company_zip + "\n" + data.company_address);
    sheet.range("B98:F101").value(
        data.note
            .replace(/(<([^>]+)>)/gi, "") // Remove html tag
            .split("\n") // Split by line
            .map((e) => e.trimStart()) // Remove leading space
            .join("\n") // Join by line
    );
    if (data.bank) {
        sheet.range("B103:F103").value(data.bank.name);
        sheet.range("B104:F104").value(`戶名：${data.bank.account_name}`);
        sheet.range("B105:F105").value(`帳號：${data.bank.account}`);
    }

    // FIXME: More then 10 products will not be displayed
    data.products.forEach((product, index) => {
        sheet.cell(`A${18 + index}`).value(index + 1);
        sheet.range(`B${18 + index}:D${18 + index}`).value(product.product);
        sheet.cell(`E${18 + index}`).value(product.qty);
        sheet.cell(`G${18 + index}`).value(product.price);
    });

    for (let i = data.products.length + 18; i <= 97; i++) {
        sheet.row(i).hidden(true);
    }
    sheet
        .range(`A${data.products.length + 17}:H${data.products.length + 17}`)
        .style({ bottomBorder: true });
}

async function bom2(workbook, data) {
    function nextChar(c, n) {
        var i = (parseInt(c, 36) + n) % 36;
        return (!i * 10 + i).toString(36).toUpperCase();
    }

    const sheet = workbook.sheet(0);
    sheet.range("C1:E1").value(data.order_code);
    sheet.range("C2:E2").value(data.name);
    sheet.cell("G1").value(data.date);
    sheet.range("C3:E3").value(data.customer);
    sheet.range("G2:G3").value(data.engineer);

    data.bom_lines.forEach((line, index) => {
        if (line.purchase) {
            sheet.range(`B${9 + index}:W${9 + index}`).style("fill", "#FFD864");
        } else {
            sheet.cell(`E${9 + index}`).value("廠內尚有庫存");
        }
        sheet.cell(`B${9 + index}`).value(line.product);
        sheet.cell(`F${9 + index}`).value(line.qty);
        if (line.best_supplier) {
            sheet.cell(`G${9 + index}`).value(line.best_supplier.name);
            sheet.cell(`H${9 + index}`).value(line.best_supplier.price);
            sheet.cell(`I${9 + index}`).value(line.best_supplier.moq);
            sheet.cell(`J${9 + index}`).value(line.best_supplier.lead_time);
            sheet.cell(`K${9 + index}`).value(line.suppliers.length);
            line.suppliers.forEach((supplier, s_idx) => {
                sheet
                    .cell(`${nextChar("L", 4 * s_idx)}${9 + index}`)
                    .value(supplier.name);
                sheet
                    .cell(`${nextChar("N", 4 * s_idx)}${9 + index}`)
                    .value(supplier.moq);
                sheet
                    .cell(`${nextChar("O", 4 * s_idx)}${9 + index}`)
                    .value(supplier.price);
            });
        }
    });

    for (let i = data.bom_lines.length + 9; i <= 88; i++) {
        sheet.row(i).hidden(true);
    }
}

async function bom1(workbook, data) {
    const sheet = workbook.sheet(0);

    sheet.range("C4:F4").value(data.project);
    sheet.range("C5:F5").value(data.inquiry_code);
    sheet.cell("H4").value(data.date);
    sheet.cell("H5").value(data.engineer);
    // FIXME: More then 10 products will not be displayed
    data.bom_lines.forEach((product, index) => {
        sheet.range(`B${8 + index}:D${8 + index}`).value(product.product);
        sheet.cell(`E${8 + index}`).value(product.qty);
        sheet.cell(`G${8 + index}`).value(product.price);
    });

    for (let i = data.bom_lines.length + 8; i <= 87; i++) {
        sheet.row(i).hidden(true);
    }
}

async function bomFetch() {
    const url = new URL(browser.location.href.replace("#", "?"));
    const id = url.searchParams.get("id");
    const res = await fetch(`/mrp/bom/dump?id=${id}`);
    const data = await res.json();
    if (data.bom_status == "external") {
        data.path = "bom2";
    } else if (data.bom_status == "internal") {
        data.path = "bom1";
    }
    return data;
}

async function bom(workbook, data) {
    if (data.bom_status == "external") {
        await bom2(workbook, data);
    } else if (data.bom_status == "internal") {
        await bom1(workbook, data);
    }
}

async function purchaseFetch() {
    const url = new URL(browser.location.href.replace("#", "?"));
    const id = url.searchParams.get("id");
    const res = await fetch(`/purchase/order/dump?id=${id}`);
    const data = await res.json();
    return data;
}

async function purchase(workbook, data) {
    const sheet = workbook.sheet(0);

    sheet.range("C3:E3").value(data.inquiry_code);
    sheet.range("C5:E5").value(data.date);
    sheet.range("A6:D6").value(data.partner);
    sheet.range("B7:D7").value(data.contract_name);
    sheet.range("B8:D8").value(data.partner_vat);
    sheet.range("B10:D10").value(data.contract_phone);
    sheet.range("B11:D11").value(data.contract_email);
    sheet
        .range("B12:D12")
        .value(data.partner_zip + "\n" + data.partner_address);

    // FIXME: More then 5 products will not be displayed
    data.products.forEach((product, index) => {
        sheet.range(`B${14 + index}:D${14 + index}`).value(product.product);
        sheet.cell(`E${14 + index}`).value(product.qty);
        sheet.cell(`G${14 + index}`).value(product.price);
    });

    for (let i = data.products.length + 14; i <= 93; i++) {
        sheet.row(i).hidden(true);
    }
    sheet
        .range(`A${data.products.length + 13}:H${data.products.length + 13}`)
        .style({ bottomBorder: "medium" });
}

class XlsxReporter {
    constructor() {
        let modelConfig = this.getModelConfig();
        this.fetchData = modelConfig.fetchData;
        this.fillBook = modelConfig.fillBook;
        this.filePath = modelConfig.path;
    }

    getModelConfig() {
        const urlParams = new URLSearchParams(browser.location.href);
        const model = urlParams.get("model");
        return modelMapping[model];
    }

    save() {
        this.workbook
            .outputAsync()
            .then(function (blob) {
                if (window.navigator && window.navigator.msSaveOrOpenBlob) {
                    window.navigator.msSaveOrOpenBlob(blob, "out.xlsx");
                } else {
                    var url = window.URL.createObjectURL(blob);
                    var a = document.createElement("a");
                    document.body.appendChild(a);
                    a.href = url;
                    a.download = "out.xlsx";
                    a.click();
                    window.URL.revokeObjectURL(url);
                    document.body.removeChild(a);
                }
            })
            .catch(function (err) {
                alert(err.message || err);
                throw err;
            });
    }

    async readFile() {
        const res = await browser.fetch(
            `/kci-report-template/static/xlsx/${this.filePath}.xlsx`
        );
        const blob = await res.blob();
        return await XlsxPopulate.fromDataAsync(blob);
    }

    async report() {
        this.data = await this.fetchData();
        // In some cases, path must read from remote
        if (this.data.path) {
            this.filePath = this.data.path;
        }
        this.workbook = await this.readFile();
        await this.fillBook(this.workbook, this.data);
        this.save();
    }
}

function reportXlsx() {
    const reporter = new XlsxReporter();
    reporter.report();
}
