import json

from odoo import http

class Quotation(http.Controller):
    @http.route('/sale/quotation/dump', type='http', auth='user')
    def index(self, **kw):
        def ensure_str(s):
            return s if s else ''

        res = {}
        quotation = http.request.env['sale.order'].search([('id', '=', kw.get('id'))])
        res['inquiry_code'] = ensure_str(quotation.inquiry_id.code)
        res['company'] = ensure_str(quotation.partner_id.parent_id.name) if quotation.partner_id.parent_id else ''
        res['contract_name'] = ensure_str(quotation.partner_id.name)
        res['contract_mobile'] = ensure_str(quotation.partner_id.mobile)
        res['contract_phone'] = ensure_str(quotation.partner_id.phone)
        res['contract_email'] = ensure_str(quotation.partner_id.email)
        res['company_vat'] = ensure_str(quotation.partner_id.vat)
        res['company_zip'] = ensure_str(quotation.partner_id.zip)
        res['company_address'] = ensure_str(quotation.partner_id.state_id.name) + \
            ensure_str(quotation.partner_id.city) + \
            ensure_str(quotation.partner_id.street) + \
            ensure_str(quotation.partner_id.street2)
        res['products'] = []
        for product in quotation.order_line:
            res['products'].append({
                'product': ensure_str(product.name),
                'qty': ensure_str(product.product_uom_qty),
                'price': ensure_str(product.price_unit),
            })
        res['note'] = ensure_str(quotation.note)
        res['note'] += '\n付款條件 ' + \
            ensure_str(quotation.payment_term_id.name) if quotation.payment_term_id else ''
        if quotation.bank:
            res['bank'] = {
                'name': ensure_str(quotation.bank.name) + ' ' + ensure_str(quotation.bank.account_sub_bank),
                'account_name': ensure_str(quotation.bank.account_name),
                'account': ensure_str(quotation.bank.account)
            }
        return json.dumps(res)